// ignore_for_file: cancel_subscriptions, public_member_api_docs, non_constant_identifier_names,lines_longer_than_80_chars, empty_catches, avoid_dynamic_calls

import 'package:flutter_blue_plus/flutter_blue_plus.dart';
import 'package:healthrian_ble_support/healthrian_ble_support.dart';
import 'package:healthrian_ble_support_for_cardioclick/'
    'healthrian_ble_support_for_cardioclick.dart' as cardioclick;
import 'package:healthrian_ble_support_for_wearcardiot/'
    'healthrian_ble_support_for_wearcardiot.dart' as wearcardiot;
import 'package:healthrian_ble_support_for_wearecg12/'
    'healthrian_ble_support_for_wearecg12.dart' as wearecg12;
import 'package:healthrian_common_support/healthrian_common_support.dart';

class BLEManager {
  factory BLEManager() {
    return _instance;
  }

  BLEManager._internal() {
    raw.isAvailable.then(
      (isAvailable) {
        if (isAvailable) raw.setLogLevel(LogLevel.info);
      },
    );
  }

  static final BLEManager _instance = BLEManager._internal();
  final FlutterBluePlus raw = FlutterBluePlus.instance;

  final manageWearEcg12 = wearecg12.BLEManager();
  final manageCardioClick = cardioclick.BLEManager();
  final manageWearCardioT = wearcardiot.BLEManager();

  final _enable = List<bool>.generate(3, (index) => true);

  void setMode({
    List<bool> enableList = const [true, true, true],
  }) {
    assert(enableList.length >= 3, 'Length of input list must be 3');
    for (var i = 0; i < enableList.length; i++) {
      _enable[i] = enableList[i];
    }
  }

  void enableWearEcg12({bool enable = true}) {
    setMode(enableList: [true, false, false]);
  }

  void enableCardioClick({bool enable = true}) {
    setMode(enableList: [false, true, false]);
  }

  void enableWearCardioT({bool enable = true}) {
    setMode(enableList: [false, false, true]);
  }

  Future<bool> connectDevice({String preferredDeviceId = ''}) async {
    if (!_enable.contains(true)) return false;
    return _perform(
      () => manageWearEcg12.connectDevice(
        preferredDeviceId: preferredDeviceId,
      ),
      () => manageCardioClick.connectDevice(
        preferredDeviceId: preferredDeviceId,
      ),
      () => manageWearCardioT.connectDevice(
        preferredDeviceId: preferredDeviceId,
      ),
      isDeviceCheck: false,
    )!;
  }

  bool _isAbleToWearCardioT({bool isDeviceCheck = true}) =>
      (isDeviceCheck ? manageWearCardioT.device != null : true) && _enable[2];

  bool _isAbleToCardioClick({bool isDeviceCheck = true}) =>
      (isDeviceCheck ? manageCardioClick.device != null : true) && _enable[1];

  bool _isAbleToWearEcg12({bool isDeviceCheck = true}) =>
      (isDeviceCheck ? manageWearEcg12.device != null : true) && _enable[0];

  T? _perform<T>(
    T Function() functionWearEcg12,
    T Function() functionCardioClick,
    T Function() functionWearCardioT, {
    bool isDeviceCheck = true,
  }) {
    if (_isAbleToWearEcg12(isDeviceCheck: isDeviceCheck)) {
      return functionWearEcg12();
    }
    if (_isAbleToCardioClick(isDeviceCheck: isDeviceCheck)) {
      return functionCardioClick();
    }
    if (_isAbleToWearCardioT(isDeviceCheck: isDeviceCheck)) {
      return functionWearCardioT();
    }
    return null;
  }

  Future<void> disconnectDevice() async {
    await _perform(
      manageWearEcg12.disconnectDevice,
      manageCardioClick.disconnectDevice,
      manageWearCardioT.disconnectDevice,
    );
  }

  Future<void> registerCharacteristics() async {
    await _perform(
      manageWearEcg12.registerCharacteristics,
      manageCardioClick.registerCharacteristics,
      manageWearCardioT.registerCharacteristics,
    );
  }

  Future<void> initializeStreams() async {
    await _perform(
      manageWearEcg12.initializeStreams,
      manageCardioClick.initializeStreams,
      manageWearCardioT.initializeStreams,
    );
  }

  Future<void> startMeasurement() async {
    await _perform(
      manageWearEcg12.startMeasurementEcg,
      manageCardioClick.startMeasurementEcg,
      manageWearCardioT.startMeasurement,
    );
  }

  Future<void> stopMeasurement() async {
    await _perform(
      manageWearEcg12.stopMeasurementEcg,
      manageCardioClick.stopMeasurementEcg,
      manageWearCardioT.stopMeasurement,
    );
  }

  Future<void> pauseMeasurement() async {
    await _perform(
      manageWearEcg12.pauseMeasurementEcg,
      manageCardioClick.pauseMeasurementEcg,
      manageWearCardioT.pauseMeasurement,
    );
  }

  int get samplingRate {
    return _perform(
          () => manageWearEcg12.samplingRate,
          () => manageCardioClick.samplingRate,
          () => manageWearCardioT.samplingRate,
        ) ??
        250;
  }

  HolterDevices? getDevice() {
    return _perform(
      () => HolterDevices.WearECG12,
      () => HolterDevices.CardioClick,
      () => HolterDevices.WearCardioT,
    );
  }

  Future<Channels?> saveMeasurement() async {
    final res = _perform(
      () async {
        if (manageWearEcg12.streams.bufferedFilterStreamEcg == null)
          return null;
        if (await manageWearEcg12.streams.bufferedFilterStreamEcg!.isEmpty)
          return null;
        return await manageWearEcg12.streams.bufferedFilterStreamEcg?.last;
      },
      () async {
        if (manageCardioClick.streams.bufferedFilterStreamEcg == null)
          return null;
        if (await manageCardioClick.streams.bufferedFilterStreamEcg!.isEmpty)
          return null;
        return await manageCardioClick.streams.bufferedFilterStreamEcg?.last
            as Channels;
      },
      () async {
        if (manageWearCardioT.streams.bufferedFilterStreamEcg == null)
          return null;
        if (await manageWearCardioT.streams.bufferedFilterStreamEcg!.isEmpty)
          return null;
        return await manageWearCardioT.streams.bufferedFilterStreamEcg?.last;
      },
    );
    return res;
  }
}
